!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DBS NetCDF Tutorial Series - 01                                  !!!
!!! This Code is for writing 1-dimensional data to NetCDF File.      !!!
!!! Last Updated (DBS): March 01, 2022                               !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! HELPLINE:-
!!! No Input File required for running this code. 
!!! On successful execution, this code would dump "01D-Data.nc"
!!! Compilation of this code must be done with the following syntax:
!!! 	gfortran -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff -o 01_write_1d_data.x 01_write_1d_data.f90
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program DBS_NetCDF01 

	USE netcdf
	implicit none

!	Output Filename
	Character(len=*), Parameter :: FILE_NAME = "01D-Data.nc"
    
!	Below, we are defining an array a1d(in), where in = 10
!	For this, we have to define three variables: 
!	in (=10, integer), n (integer, with dimension 10)
!	a1d (real, with dimension of 10, i.e., in)
	Integer, Parameter :: in=10  
	Real, Dimension(in) :: n, a1d

	Integer :: ncid, varid
!	Below, we need to define dimension and variable id of both 'n' and 'a1d'
	Integer :: n_dimid, a1d_dimid
	Integer :: n_varid, a1d_varid 

!	Below, we need to define the variable names.
!	These names will appear as such in NetCDF heder file
	Character(len=*),Parameter :: N_NAME= "N"
	Character(len=*),Parameter :: A1D_NAME= "A1D"

!	Below are 'optional' definitions.
!	We can define as many attributes, as we want.
	Character (len=*), Parameter :: UNITS = "units"
  	Character (len=*), Parameter :: N_UNITS = "numbers"
  	Character (len=*), Parameter :: A1D_UNITS = "meters"
!	Second, Third, ..... Attributes (optional)
  	Character (len=*), Parameter :: NATT1 = "aaaa"
  	Character (len=*), Parameter :: N_AT1 = "bbbb"
  	Character (len=*), Parameter :: AATT1 = "cccc"
  	Character (len=*), Parameter :: A_AT1 = "dddd"

!	This step is 'optional'. Can be used to fill blank values.
	Real, Parameter :: fillval_a1d = -9999.9

!	Provide some database here. 
!	In reality, this part can be read from other files.
	Data n /1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0/
	Data a1d /0.0, 1.0, 2.0, 3.0, 5.0, 4.0, 3.0, 2.0, 1.0, 0.0/

!	This syntax is for opening a new file for writing purpose.
	Call check( nf90_create(FILE_NAME, nf90_clobber, ncid) )

!	Now, we initate defining the dimensions first. 
!	Here, it is 1-D dat. 
	Call check( nf90_def_dim(ncid, N_NAME, in, n_dimid) )

!	Now, we definite the variables.
!	Here, we have 'a1d' as a function of 'n', so we define two variables
	Call check( nf90_def_var(ncid, N_NAME, NF90_REAL, n_dimid, n_varid) )
	Call check( nf90_def_var(ncid, A1D_NAME, NF90_REAL, n_dimid, a1d_varid) )

!	Now, we can add attributes to variables.
!	This is 'optional'. We can add as many attributes as we want.
	Call check( nf90_put_att(ncid, n_varid, UNITS, N_UNITS) )
	Call check( nf90_put_att(ncid, n_varid, NATT1, N_AT1) )

	Call check( nf90_put_att(ncid, a1d_varid, UNITS, A1D_UNITS) )
	Call check( nf90_put_att(ncid, a1d_varid, AATT1, A_AT1) )

!	We can fill the blank values of a variable in the following way:
	Call check( nf90_put_att(ncid, a1d_varid, "_FillVlue", fillval_a1d) )

!	With the following step, we will close definition of dimension & variables.
!	Warning: We can not define a new variable of dimension after this step.
	Call check ( nf90_enddef(ncid) )

!	Here, we are writing the variables.
	Call check ( nf90_put_var(ncid, n_varid, n) )
	Call check ( nf90_put_var(ncid, a1d_varid, a1d) )

!	Bottom step will close the NetCDF File.
	Call check ( nf90_close(ncid) )

	Print *, '1D-Data is dumped successfully !!!!'

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program DBS_NetCDF01
