!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DBS NetCDF Tutorial Series - 02                                  !!!
!!! This Code is for reading 1-dimensional data from a NetCDF File.  !!!
!!! Last Updated (DBS): March 01, 2022                               !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! HELPLINE:-
!!! We will be using "01D-Data.nc" file as the input file here. 
!!! We have grabbed the following information from the NetCDF fle:
!!! ncdump 01D-Data.nc > header.txt
!!! Useful contents of 'header.txt' looks like this: 
!netcdf \01D-Data {
!dimensions:
!        N = 10 ;
!variables:
!        float N(N) ;
!        float A1D(N) ;
!}
!!! On successful execution, this code would dump "1D-Data.dat"
!!! Compilation of this code must be done with the following syntax:
!!! 	gfortran -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff -o 02_read_1d_data.x 01_read_1d_data.f90
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program DBS_NetCDF02 

	USE netcdf
	implicit none

!	Input Filename
	Character(len=*), Parameter :: FILE_NAME = "01D-Data.nc"
    
!	Below, we are defining an array a1d(in), where in = 10
!	For this, we have to define three variables: 
!	in (=10, integer), n (integer, with dimension 10)
!	a1d (real, with dimension of 10, i.e., in)
	Integer, Parameter :: in=10  
	Real, Dimension(in) :: n, a1d

	Integer :: i, ncid, varid
!	Below, we need to define dimension and variable id of both 'n' and 'a1d'
	Integer :: n_dimid, a1d_dimid
	Integer :: n_varid, a1d_varid 

!	Below, we need to define the variable names.
!	These names will appear as such in NetCDF heder file
	Character(len=*),Parameter :: N_NAME= "N"
	Character(len=*),Parameter :: A1D_NAME= "A1D"

!	This syntax is for opening an existing file for reading purpose.
	Call check( nf90_open(FILE_NAME, nf90_nowrite, ncid) )

!	Now, we start reading the variables from NetCDF file. 
	Call check( nf90_inq_varid(ncid, N_NAME, varid) )
	Call check( nf90_get_var(ncid, varid, n) )

	Call check( nf90_inq_varid(ncid, A1D_NAME, varid) )
	Call check( nf90_get_var(ncid, varid, a1d) )

!	Bottom step will close the NetCDF File.
	Call check ( nf90_close(ncid) )

	Print *, '1D-Data is read successfully !!!!'

	Open (1, file = '01D-Data.dat', form='formatted')

	Write (1, 10) 'N', 'A1D'
 10	Format (2(1x, a10))

	Do i = 1, in
	Write (1, 11) n(i), a1d(i)
 11	Format (2(1x, f10.2))
	Enddo 

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program DBS_NetCDF02
