!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DBS NetCDF Tutorial Series - 03                                  !!!
!!! This Code is for writing 2-dimensional data to NetCDF File.      !!!
!!! Last Updated (DBS): March 01, 2022                               !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! HELPLINE:-
!!! No Input File required for running this code. 
!!! On successful execution, this code would dump "2D-Data.nc"
!!! Compilation of this code must be done with the following syntax:
!!! 	gfortran -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff -o 03_write_2d_data.x 03_write_2d_data.f90
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program DBS_NetCDF03 

	USE netcdf
	implicit none

!	Output Filename
	Character(len=*), Parameter :: FILE_NAME = "02D-Data.nc"
    
!	Below, we are defining an array a2d(ix,iy), where ix=2, iy=3
!	ndims is defined for 2-dimensional array later
	Integer, Parameter :: ix=2, iy=3, ndims=2  
	Integer, Dimension(ndims) :: dimids
	Real, Dimension(ix) :: x
	Real, Dimension(iy) :: y 
	Real, Dimension(ix,iy) :: a2d

	Integer :: ncid, varid
!	Below, we need to define dimension and variable id of 'x', 'y' and 'a2d'
	Integer :: x_dimid, y_dimid, a2d_dimid
	Integer :: x_varid, y_varid, a2d_varid 

!	Below, we need to define the variable names.
!	These names will appear as such in NetCDF heder file
	Character(len=*),Parameter :: X_NAME= "X"
	Character(len=*),Parameter :: Y_NAME= "Y"
	Character(len=*),Parameter :: A2D_NAME= "A2D"

!	Below are 'optional' definitions.
!	We can define as many attributes, as we want.
	Character (len=*), Parameter :: UNITS = "units"
  	Character (len=*), Parameter :: X_UNITS = "X numbers"
  	Character (len=*), Parameter :: Y_UNITS = "Y numbers"
  	Character (len=*), Parameter :: A2D_UNITS = "meters"

!	Provide some database here. 
!	In reality, this part can be read from other files.
	Data x /1.0, 2.0/
	Data y /3.0, 4.0, 5.0/
	Data a2d /11.0, 12.0, 13.0, 14.0, 15.0, 16.0/

!	This syntax is for opening a new file for writting purpose.
	Call check( nf90_create(FILE_NAME, nf90_clobber, ncid) )

!	Now, we initate defining the dimensions first. 
!	Here, it is 2-D dat. 
	Call check( nf90_def_dim(ncid, X_NAME, ix, x_dimid) )
	Call check( nf90_def_dim(ncid, Y_NAME, iy, y_dimid) )

!	Now, we definite the variables.
!	Here, we have 'a2d' as a function of 'x' and 'y', 
!	So we firtly define 'x' and 'y' as one-dimensional variables'
	Call check( nf90_def_var(ncid, X_NAME, NF90_REAL, x_dimid, x_varid) )
	Call check( nf90_def_var(ncid, Y_NAME, NF90_REAL, y_dimid, y_varid) )
!	Now, we define 'a2d' as two-dimensional variable:
	dimids = (/ x_dimid, y_dimid /)
	Call check( nf90_def_var(ncid, A2D_NAME, NF90_REAL, dimids, a2d_varid) )

!	Now, we can add attributes to variables.
!	This is 'optional'. We can add as many attributes as we want.
	Call check( nf90_put_att(ncid, x_varid, UNITS, X_UNITS) )
	Call check( nf90_put_att(ncid, y_varid, UNITS, Y_UNITS) )
	Call check( nf90_put_att(ncid, a2d_varid, UNITS, A2D_UNITS) )

!	With the following step, we will close definition of dimension & variables.
!	Warning: We can not define a new variable of dimension after this step.
	Call check ( nf90_enddef(ncid) )

!	Here, we are writing the variables.
	Call check ( nf90_put_var(ncid, x_varid, x) )
	Call check ( nf90_put_var(ncid, y_varid, y) )
	Call check ( nf90_put_var(ncid, a2d_varid, a2d) )

!	Bottom step will close the NetCDF File.
	Call check ( nf90_close(ncid) )

	Print *, '2D-Data is dumped successfully !!!!'

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program DBS_NetCDF03
