!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DBS NetCDF Tutorial Series - 04                                  !!!
!!! This Code is for reading 2-dimensional data from a NetCDF File.  !!!
!!! Last Updated (DBS): March 01, 2022                               !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! HELPLINE:-
!!! We will be using "02D-Data.nc" file as the input file here. 
!!! We have grabbed the following information from the NetCDF fle:
!!! ncdump 01D-Data.nc > header.txt
!!! Useful contents of 'header.txt' looks like this: 
!netcdf \02D-Data {
!dimensions:
!        X = 2 ;
!        Y = 3 ;
!variables:
!        float X(X) ;
!        float Y(Y) ;
!        float A2D(Y, X) ;
!}
!!! On successful execution, this code would dump "1D-Data.dat"
!!! Compilation of this code must be done with the following syntax:
!!! 	gfortran -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff -o 04_read_2d_data.x 04_read_2d_data.f90
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program DBS_NetCDF04 

	USE netcdf
	implicit none

!	Input Filename
	Character(len=*), Parameter :: FILE_NAME = "02D-Data.nc"
    
!	Below, we are defining an array a2d(ix,iy), where ix=2, iy=3
	Integer, Parameter :: ix=2, iy=3  
	Real, Dimension(ix) :: x
	Real, Dimension(iy) :: y
	Real, Dimension(ix,iy) :: a2d

	Integer :: i, j, ncid, varid
!	Below, we need to define dimension and variable id of 'x', 'y', and 'a2d'
	Integer :: x_dimid, y_dimid, a2d_dimid
	Integer :: x_varid, y_varid, a2d_varid 

!	Below, we need to define the variable names.
!	These names will appear as such in NetCDF heder file
	Character(len=*),Parameter :: X_NAME= "X"
	Character(len=*),Parameter :: Y_NAME= "Y"
	Character(len=*),Parameter :: A2D_NAME= "A2D"

!       This syntax is for opening an existing file for reading purpose.
	Call check( nf90_open(FILE_NAME, nf90_nowrite, ncid) )

!	Now, we start reading the variables from NetCDF file. 
	Call check( nf90_inq_varid(ncid, X_NAME, varid) )
	Call check( nf90_get_var(ncid, varid, x) )

	Call check( nf90_inq_varid(ncid, Y_NAME, varid) )
	Call check( nf90_get_var(ncid, varid, y) )

	Call check( nf90_inq_varid(ncid, A2D_NAME, varid) )
	Call check( nf90_get_var(ncid, varid, a2d) )

!	Bottom step will close the NetCDF File.
	Call check ( nf90_close(ncid) )

	Print *, '2D-Data is read successfully !!!!'

	Open (1, file = '02D-Data.dat', form='formatted')

	Write (1, 10) 'ix', 'iy', 'X(ix)', 'Y(iy)', 'A2D(ix,iy)'
 10	Format (5(1x, a10))

	Do i = 1, ix
	Do j = 1, iy
	Write (1, 11) i, j, x(i), y(j), a2d(i,j)
 11	Format (2(1x, i10), 3(1x, f10.2))
	Enddo 
	Enddo 

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program DBS_NetCDF04
