!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DBS NetCDF Tutorial Series - 05                                  !!!
!!! This Code is for appending new data to an existing NetCDF File.  !!!
!!! Last Updated (DBS): March 01, 2022                               !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! HELPLINE:-
!!! Any NetCDF File is required for running this code.
!!! It is assumed that the existing NetCDF file does not have any 
!!! variable named: rad_sw_in, rad_lw_in, and time_rad 
!!! On successful execution, this code would append data to NC file.
!!! Compilation of this code must be done with the following syntax:
!!! 	gfortran -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff -o 05_append_ncfile.x 05_append_ncfile.f90
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program DBS_NetCDF05

	USE netcdf
	implicit none

!	Precaution: You can not run this code two times.
!	After first execution, this code will append 3 new variables:
!	time_rad, rad_sw_in, and rad_lw_in
!	So it is advisable to keep a backup of original NC file with a different name.
!	THIS CODE WILL OVERWRITE 'INIFOR-DYNAMIC-DRIVER.nc' FILE
	Character(len=*), Parameter :: FILE_NAME = "INIFOR-DYNAMIC-DRIVER.nc"
    
	Integer, Parameter	:: itime_rad=25

	Real, Dimension(itime_rad) :: time_rad, rad_sw_in, rad_lw_in

	Integer	:: ncid, varid, time_raddimid, rad_sw_indimid, rad_lw_indimid

	Character(len=*),Parameter :: TIME_RAD_NAME="time_rad"
	Character(len=*),Parameter :: RAD_SW_IN_NAME= "rad_sw_in"
	Character(len=*),Parameter :: RAD_LW_IN_NAME= "rad_lw_in"

	Character (len = *), parameter :: UNITS = "units"
  	Character (len = *), parameter :: TIME_RAD_UNITS = "seconds"
  	Character (len = *), parameter :: RAD_SW_IN_UNITS = "W m-2"
  	Character (len = *), parameter :: RAD_LW_IN_UNITS = "W m-2"

	Character (len = *), parameter :: LNAME = "long_name"
	Character (len = *), parameter :: SNAME = "standard_name"
  	Character (len = *), parameter :: TIME_RAD_LNAME = "time"
  	Character (len = *), parameter :: RAD_SW_IN_SNAME = "sobs"
  	Character (len = *), parameter :: RAD_LW_IN_SNAME = "thbs"

  	Character (len = *), parameter :: RAD_SW_IN_LOD = "1"
  	Character (len = *), parameter :: RAD_LW_IN_LOD = "1"

	Real, Parameter :: fillval_rad_sw_in = -9999.9
	Real, Parameter :: fillval_rad_lw_in = -9999.9

	Integer :: time_rad_varid, rad_sw_in_varid, rad_lw_in_varid
	Integer :: time_rad_dimid, rad_sw_in_dimid, rad_lw_in_dimid

	Integer, Parameter :: lod_rad = 1

	Data time_rad /    0.0,  3600.0,  7200.0, 10800.0, 14400.0,     &
                       18000.0, 21600.0, 25200.0, 28800.0, 32400.0,     &
                       36000.0, 39600.0, 43200.0, 46800.0, 50400.0,     &
                       54000.0, 57600.0, 61200.0, 64800.0, 68400.0,     &
                       72000.0, 75600.0, 79200.0, 82800.0, 86400.0/

	Data rad_sw_in /  0.000,   1.238,  53.411, 167.530, 280.220,    & 
                        201.731,  32.576, 119.458, 381.052,  53.909,    &
                        100.378,  31.617,   6.485,   0.000,   0.000,    &
                          0.000,   0.000,   0.000,   0.000,   0.000,    &
                          0.000,   0.000,   0.000,   0.000,   0.000/

	Data rad_lw_in /-10.976,  -8.337, -13.322, -22.936, -32.622,    &
                        -32.675, -26.956, -16.615, -33.552, -30.965,    &
                        -23.504, -21.203, -18.733, -16.575, -15.722,    &
                        -18.344, -19.718, -19.203, -19.982, -19.412,    &
                        -22.512, -17.958, -17.514, -17.459, -12.195/

	Call check( nf90_open(FILE_NAME, nf90_write, ncid) )

	Call check( nf90_redef(ncid) )
  
	Call check( nf90_def_dim(ncid, TIME_RAD_NAME, itime_rad, time_rad_dimid) )

	Call check( nf90_def_var(ncid, TIME_RAD_NAME, NF90_REAL, time_rad_dimid, time_rad_varid) )
	Call check( nf90_def_var(ncid, RAD_SW_IN_NAME, NF90_REAL, time_rad_dimid, rad_sw_in_varid) )
	Call check( nf90_def_var(ncid, RAD_LW_IN_NAME, NF90_REAL, time_rad_dimid, rad_lw_in_varid) )

	Call check( nf90_put_att(ncid, time_rad_varid, UNITS, TIME_RAD_UNITS) )
	Call check( nf90_put_att(ncid, rad_sw_in_varid, UNITS, RAD_SW_IN_UNITS) )
	Call check( nf90_put_att(ncid, rad_lw_in_varid, UNITS, RAD_LW_IN_UNITS) )

	Call check( nf90_put_att(ncid, time_rad_varid, LNAME, TIME_RAD_LNAME) )
	Call check( nf90_put_att(ncid, rad_sw_in_varid, SNAME, RAD_SW_IN_SNAME) )
	Call check( nf90_put_att(ncid, rad_lw_in_varid, SNAME, RAD_LW_IN_SNAME) )

	Call check( nf90_put_att(ncid, rad_sw_in_varid, "_FillValue", fillval_rad_sw_in) )
	Call check( nf90_put_att(ncid, rad_lw_in_varid, "_FillValue", fillval_rad_lw_in) )

	Call check( nf90_put_att(ncid, rad_sw_in_varid, "lod", lod_rad) )
	Call check( nf90_put_att(ncid, rad_lw_in_varid, "lod", lod_rad) )

	Call check ( nf90_enddef(ncid) )

	Call check ( nf90_put_var(ncid, time_rad_varid, time_rad) )
	Call check ( nf90_put_var(ncid, rad_sw_in_varid, rad_sw_in) )
	Call check ( nf90_put_var(ncid, rad_lw_in_varid, rad_lw_in) )

	Call check ( nf90_close(ncid) )

	Print *, 'Sample NetCDF File is appended with new data successfully !!'

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program DBS_NetCDF05 
